//***************************************************************************
//
//   TChartUndoRedo component for TeeChartPro VCL
//        Sample application version 1.02
//      (c)2004 Ivo Ungermann
//   http://www.dcentral.name/delphi/components
//   mailto://projects@dcentral.name
//
//   Licensed according to GNU GPL version 2
//     http://www.gnu.org/licenses/gpl.html
//
//***************************************************************************

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TeEngine, Series, ExtCtrls, ActnList, TeeProcs, Chart, StdCtrls, Buttons, TeeEdit,
  Menus, ImgList, ChartUndoRedo, Spin;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    PosLabel: TLabel;
    Button1: TButton;
    ChartEditor1: TChartEditor;
    ActionList1: TActionList;
    UA: TAction;
    RA: TAction;
    Series1: TFastLineSeries;
    ImageList1: TImageList;
    Series2: TFastLineSeries;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Label2: TLabel;
    Label3: TLabel;
    DepthLabel: TLabel;
    Button2: TButton;
    Label1: TLabel;
    ChartUndoRedo1: TChartUndoRedo;
    UndoLevelsSpin: TSpinEdit;
    Label4: TLabel;
    Label5: TLabel;
    UndoLevelsLabel: TLabel;
    ApplyBtn: TButton;
    GroupScrollBox: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ChartUndoRedo1SaveUndoPos(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure UndoLevelsSpinChange(Sender: TObject);
    procedure ApplyBtnClick(Sender: TObject);
    procedure GroupScrollBoxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
//browser-style undo/redo shortcuts
UA.ShortCut := Shortcut(VK_LEFT, [ssAlt] );
RA.ShortCut := Shortcut(VK_RIGHT, [ssAlt] );
//after creating (loading etc.) chart, initialize undo buffer and save first position
ChartUndoRedo1.ClearUndoBuffer;
ChartUndoRedo1.SaveUndoPos;
//actualize labels and user interface
UndoLevelsSpin.Value := ChartUndoRedo1.UndoLevels;
ApplyBtn.Enabled := False;
GroupScrollBox.Checked := ChartUndoRedo1.GroupScroll;
UndoLevelsLabel.Caption := IntToStr(ChartUndoRedo1.UndoLevels);
end;

procedure TForm1.ChartUndoRedo1SaveUndoPos(Sender: TObject);
begin
//informational labels
PosLabel.Caption := IntToStr(ChartUndoRedo1.UndoRedoPos);
DepthLabel.Caption := IntToStr(ChartUndoRedo1.UndoRedoMax + 1);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
//chart editing is not tracked automatically, save position manually
ChartEditor1.Execute;
ChartUndoRedo1.SaveUndoPos;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i, j: Integer;
begin
//another action
//add some random points
j := Random(5) + 3;
for i := 1 to j do with Chart1 do
  begin
  //cannot use original serie names (Series1, Series2) -  series object is
  // not the same after undo/redo operation
  Series[0].Add(Random(100) - 50 + Series[0].YValue[Series[0].Count - 1]);
  Series[1].Add(Random(100) - 50 + Series[1].YValue[Series[1].Count - 1]);
  end;
//save position manually - only if you want track data changes
ChartUndoRedo1.SaveUndoPos;
end;

procedure TForm1.UndoLevelsSpinChange(Sender: TObject);
begin
//enable apply changes
ApplyBtn.Enabled := True;
end;

procedure TForm1.ApplyBtnClick(Sender: TObject);
begin
//apply changes and disable button
ChartUndoRedo1.UndoLevels := UndoLevelsSpin.Value;
ApplyBtn.Enabled := False;
//save again initial position if buffer cleared (undo levels was decreased)
if ChartUndoRedo1.UndoRedoMax = -1 then
  ChartUndoRedo1.SaveUndoPos;
//actualize informational labels
UndoLevelsLabel.Caption := IntToStr(ChartUndoRedo1.UndoLevels);
PosLabel.Caption := IntToStr(ChartUndoRedo1.UndoRedoPos);
DepthLabel.Caption := IntToStr(ChartUndoRedo1.UndoRedoMax + 1);
end;

procedure TForm1.GroupScrollBoxClick(Sender: TObject);
begin
ChartUndoRedo1.GroupScroll := GroupScrollBox.Checked;
//save again initial position if buffer cleared (group scroll status changed)
if ChartUndoRedo1.UndoRedoMax = -1 then
  ChartUndoRedo1.SaveUndoPos;
end;

end.
